function plot_wavelet(M,Jmin)

% plot_wavelet - plot 1d or 2D wavelet transform stored using Mallat's ordering.
%
% adapted from Gabriel Peyre
% http://www.ceremade.dauphine.fr/~peyre/numerical-tour/tours/

if (nargin<2)
    Jmin = 1;
end
M = double(M);
edge_color = 'r';
lw = 2;

n = size(M,1);
Jmax = log2(n)-1;

if size(M,1)==1 || size(M,2)==1
    % 1D Signal
    hold on;
    h = plot(M); axis tight;
    a = min(M); b = max(M);
    for j=Jmin:Jmax
        plot(2^j*[1 1], [a b], 'r--');
    end
    hold off;
    %box on;
    return;
end

for j=Jmin:Jmax
    qmin = double(~(j==Jmin));
    for q=qmin:3
        [selx,sely] = compute_quadsel(j,q);
        M1 = M(selx,sely,:);
        if q>0
            st = std(M1(:));
            if st<eps
                st=1;
            end
            M1 = 0.3 * M1/st;
            M1 = max(M1,-1);
            M1 = min(M1,1);
            M1 = (M1+1)/2; % rescale( M1 );
        else
            mini = min(M1(:));
            Maxi = max(M1(:));
            a = 0;
            b = 1;
            M1 = (b-a) * (M1-mini)/(Maxi-mini) + a;
        end
        M(selx,sely,:) = M1;
    end
end

hold on;
s = [1/2 n-1/2];
imagesc(s,s,M);
axis image; axis off;
colormap gray(256);
% display axis separation
for j=Jmax:-1:Jmin;
    x = [0 2^(j+1)];
    y = [2^j 2^j];
    h = plot(x,y, edge_color);
    set(h, 'LineWidth', lw);
    y = [0 2^(j+1)];
    x = [2^j 2^j];
    h = plot(x,y, edge_color);
    set(h, 'LineWidth', lw);
end
% plot overall square
x = [0 0 n n 0];
y = [0 n n 0 0];
h = plot(x,y,edge_color);
set(h, 'LineWidth', lw*1.2);
hold off;
axis ij;



    function [selx,sely] = compute_quadsel(j,q)
        
        % compute_quadsel - compute the indices for selecting subband of a wavelet transform.
        %
        %   [selx,sely] = compute_quadrant_selection(j,q, options);
        %
        %   q=1 is for phi/psi (on X / on Y)
        %   q=2 is for psi/phi (on X / on Y)
        %   q=3 is for psi/psi (on X / on Y)
        %
        %   % Suppose that MW contains a wavelet transform
        %   j = 4; q = 2;
        %   [selx,sely] = compute_quadrant_selection(j,q);
        %   MWs = MW(selx,sely);    % MWs contains the quadrant.
        %   Copyright (c) 2004 Gabriel Peyre
        
        if q<0 || q>3
            error('Quadrant number is in {0,1,2,3}.');
        end
        if q==0
            selx = 1:2^j;
            sely = 1:2^j;
        elseif q==1 % 1st quadrant
            selx = 1:2^j;
            sely = (2^j+1):2^(j+1);
        elseif q==2
            selx = (2^j+1):2^(j+1);
            sely = 1:2^j;
        elseif q==3
            selx = (2^j+1):2^(j+1);
            sely = (2^j+1):2^(j+1);
        else
            selx = []; sely = [];
        end
    end
end

